@@ -682,7 +682,10 @@ entry_activated_callback (GtkWidget *widget, IconSelectionData *selection_data)
 	}
 }
 
-/* here's the actual routine that creates the icon selector */
+/* here's the actual routine that creates the icon selector.
+   Note that this may return NULL if the dialog was destroyed before the
+   icons were all loaded. GnomeIconSelection reenters the main loop while
+   it loads the icons, so beware! */
 
 GtkWidget *
 eel_gnome_icon_selector_new (const char *title,
@@ -737,8 +740,6 @@ eel_gnome_icon_selector_new (const char *title,
 		gnome_icon_selection_add_directory (GNOME_ICON_SELECTION (icon_selection), icon_directory);
 	}
 	
-	gnome_icon_selection_show_icons (GNOME_ICON_SELECTION (icon_selection));
-
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (dialog_response_callback), selection_data);
 
@@ -749,6 +750,15 @@ eel_gnome_icon_selector_new (const char *title,
 	g_signal_connect (entry, "activate",
 			  G_CALLBACK (entry_activated_callback), selection_data);
 
+	/* We add a weak pointer to the dialog, so we know if it has been
+	   destroyed while the icons are being loaded. */
+	eel_add_weak_pointer (&dialog);
+
+	gnome_icon_selection_show_icons (GNOME_ICON_SELECTION (icon_selection));
+
+	/* Now remove the weak pointer, if the dialog still exists. */
+	eel_remove_weak_pointer (&dialog);
+
 	return dialog;
 }
 
