@@ -99,6 +99,8 @@ static void          eel_labeled_image_class_init         (EelLabeledImageClass
 static void          eel_labeled_image_init               (EelLabeledImage       *image);
 static void          eel_labeled_image_finalize           (GObject               *object);
 
+
+
 /* GObjectClass methods */
 static void          eel_labeled_image_set_property       (GObject               *object,
 							   guint                  property_id,
@@ -108,6 +110,10 @@ static void          eel_labeled_image_get_property       (GObject
 							   guint                  property_id,
 							   GValue                *value,
 							   GParamSpec            *pspec);
+
+/* GtkObjectClass methods */
+static void          eel_labeled_image_destroy            (GtkObject             *object);
+
 /* GtkWidgetClass methods */
 static void          eel_labeled_image_size_request       (GtkWidget             *widget,
 							   GtkRequisition        *requisition);
@@ -149,6 +155,7 @@ static void
 eel_labeled_image_class_init (EelLabeledImageClass *labeled_image_class)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (labeled_image_class);
+	GtkObjectClass *object_class = GTK_OBJECT_CLASS (labeled_image_class);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (labeled_image_class);
 	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (labeled_image_class);
 	GtkBindingSet *binding_set;
@@ -158,7 +165,10 @@ eel_labeled_image_class_init (EelLabeledImageClass *labeled_image_class)
 	/* GObjectClass */
 	gobject_class->set_property = eel_labeled_image_set_property;
 	gobject_class->get_property = eel_labeled_image_get_property;
-	
+
+	/* GtkObjectClass */
+	object_class->destroy = eel_labeled_image_destroy;
+
  	/* GtkWidgetClass */
  	widget_class->size_request = eel_labeled_image_size_request;
 	widget_class->size_allocate = eel_labeled_image_size_allocate;
@@ -316,6 +326,24 @@ eel_labeled_image_finalize (GObject *object)
 }
 
 
+static void
+eel_labeled_image_destroy (GtkObject *object)
+{
+ 	EelLabeledImage *labeled_image;
+	
+	labeled_image = EEL_LABELED_IMAGE (object);
+
+	if (labeled_image->details->image != NULL) {
+		gtk_widget_destroy (labeled_image->details->image);
+	}
+
+	if (labeled_image->details->label != NULL) {
+		gtk_widget_destroy (labeled_image->details->label);
+	}
+
+	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+}
+
 /* GObjectClass methods */
 static void
 eel_labeled_image_set_property (GObject      *object,
@@ -638,13 +666,13 @@ eel_labeled_image_forall (GtkContainer *container,
 	g_return_if_fail (EEL_IS_LABELED_IMAGE (container));
 	g_return_if_fail (callback != NULL);
 	
-	labeled_image = EEL_LABELED_IMAGE (container);;
-	
+	labeled_image = EEL_LABELED_IMAGE (container);
+
 	if (include_internals) {
 		if (labeled_image->details->image != NULL) {
 			(* callback) (labeled_image->details->image, callback_data);
 		}
-
+		
 		if (labeled_image->details->label != NULL) {
 			(* callback) (labeled_image->details->label, callback_data);
 		}
