@@ -27,7 +27,10 @@
 #include <config.h>
 #include "eel-pango-extensions.h"
 
+#if !defined (EEL_OMIT_SELF_CHECK)
 #include "eel-lib-self-check-functions.h"
+#endif
+
 #include <gtk/gtkmain.h>
 #include <pango/pango-attributes.h>
 #include <pango/pango-break.h>
@@ -496,9 +499,9 @@ eel_pango_layout_set_text_ellipsized (PangoLayout  *layout,
  * Return value: the resulting dimensions.
  **/
 PangoRectangle
-eel_pango_layout_fit_to_dimensions (PangoLayout  *layout,
-				    int           max_width,
-				    int           max_height)
+eel_pango_layout_fit_to_dimensions (PangoLayout *layout,
+				    int          max_width,
+				    int          max_height)
 {
 	gint size;
 	PangoContext *context;
@@ -508,18 +511,23 @@ eel_pango_layout_fit_to_dimensions (PangoLayout  *layout,
 	g_return_val_if_fail (PANGO_IS_LAYOUT (layout), logical_rect);
 
 	context = pango_layout_get_context (layout);
-	font_desc = pango_context_get_font_description (context);
-
+	font_desc = pango_font_description_copy (pango_context_get_font_description (context));
 	size = pango_font_description_get_size (font_desc);
 
-	do {
-		pango_font_description_set_size (font_desc, size--);
-
+	while (size > 0) {
+		pango_font_description_set_size (font_desc, size);
+		pango_layout_set_font_description (layout, font_desc);
 		pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
-	} while (((max_width > 0 && logical_rect.width > max_width) || max_width < 0) &&
-		 ((max_height > 0 && logical_rect.height > max_height) || max_height < 0));
+		if (!((((max_width > 0 && logical_rect.width > max_width) || max_width < 0) &&
+		       ((max_height > 0 && logical_rect.height > max_height) || max_height < 0))))
+			break;
+		
+		size -= PANGO_SCALE / 2;
+	}
 
+	pango_font_description_free (font_desc);
+	
 	return logical_rect;
 }
 
