@@ -50,9 +50,6 @@ struct EelPasswordDialogDetails
 	GtkWidget *remember_button;
 };
 
-/* Dialog button indices */
-static const gint  DIALOG_OK_BUTTON = 0;
-
 /* Caption table rows indices */
 static const guint CAPTION_TABLE_USERNAME_ROW = 0;
 static const guint CAPTION_TABLE_PASSWORD_ROW = 1;
@@ -121,9 +118,14 @@ dialog_show_callback (GtkWidget *widget, gpointer callback_data)
 
 	password_dialog = EEL_PASSWORD_DIALOG (callback_data);
 
-	/* Move the focus to the password entry */
-	eel_caption_table_entry_grab_focus (EEL_CAPTION_TABLE (password_dialog->details->table), 
-					    CAPTION_TABLE_PASSWORD_ROW);
+	if (password_dialog->details->readonly_username == FALSE) {
+		eel_caption_table_entry_grab_focus (EEL_CAPTION_TABLE (password_dialog->details->table), 
+						    CAPTION_TABLE_USERNAME_ROW);
+	}
+	else {
+		eel_caption_table_entry_grab_focus (EEL_CAPTION_TABLE (password_dialog->details->table), 
+						    CAPTION_TABLE_PASSWORD_ROW);
+	}
 }
 
 static void
@@ -230,7 +232,7 @@ eel_password_dialog_new (const char	*dialog_title,
 			    TRUE,	/* expand */
 			    TRUE,	/* fill */
 			    0);		/* padding */
-#if 0 /* FIXME: disabled for PR2 originally, when will we re-enable */
+#if 0 /* FIXME: disabled for PR2 originally, when will we re-enable? */
 	password_dialog->details->remember_button = 
 		gtk_check_button_new_with_label (_("Remember this password"));
 
@@ -298,6 +300,8 @@ eel_password_dialog_set_readonly_username (EelPasswordDialog	*password_dialog,
 {
 	g_return_if_fail (password_dialog != NULL);
 	g_return_if_fail (EEL_IS_PASSWORD_DIALOG (password_dialog));
+
+	password_dialog->details->readonly_username = readonly;
 	
 	eel_caption_table_set_entry_readonly (EEL_CAPTION_TABLE (password_dialog->details->table),
 						   CAPTION_TABLE_USERNAME_ROW,
